------------------------------------------------------------------------------
--                                                                          --
--                         GNAT COMPILER COMPONENTS                         --
--                                                                          --
--                                S I N F O                                 --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                            $Revision: 1.146 $                            --
--                                                                          --
--           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          --
--                                                                          --
-- GNAT is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNAT;  see file COPYING.  If not, write --
-- to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. --
--                                                                          --
------------------------------------------------------------------------------

with Atree; use Atree;

package body Sinfo is

   use Atree.Unchecked_Access;
   --  This package is one of the few packages which is allowed to make direct
   --  references to tree nodes (since it is in the business of providing a
   --  higher level of tree access which other clients are expected to use and
   --  which implements checks).

   use Atree_Private_Part;
   --  The only reason that we ask for direct access to the private part of
   --  the tree package is so that we can directly reference the Nkind field
   --  of nodes table entries. We do this since it helps the efficiency of
   --  the Sinfo debugging checks considerably (note that when we are checking
   --  Nkind values, we don't need to check for a valid node reference, because
   --  we will check that anyway when we reference the field).

   NT : Nodes.Table_Ptr renames Nodes.Table;
   --  A short hand abbreviation, useful for the debugging checks

   ----------------------------
   -- Field Access Functions --
   ----------------------------

   function Abort_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Requeue_Statement);
      return Flag1 (N);
   end Abort_Present;

   function Abortable_Part
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Asynchronous_Select);
      return Node2 (N);
   end Abortable_Part;

   function Abstract_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Derived_Type_Definition
          or else NT (N).Nkind = N_Formal_Private_Type_Definition
          or else NT (N).Nkind = N_Private_Extension_Declaration
          or else NT (N).Nkind = N_Private_Type_Declaration
          or else NT (N).Nkind = N_Record_Definition);
      return Flag4 (N);
   end Abstract_Present;

   function Accept_Statement
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Alternative);
      return Node2 (N);
   end Accept_Statement;

   function Actions
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Expression_Actions);
      return List1 (N);
   end Actions;

   function Activation_Chain_Entity
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Generic_Package_Declaration
          or else NT (N).Nkind = N_Package_Declaration
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Task_Body);
      return Node2 (N);
   end Activation_Chain_Entity;

   function Acts_As_Spec
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit
          or else NT (N).Nkind = N_Subprogram_Body);
      return Flag4 (N);
   end Acts_As_Spec;

   function Actual_Parameter
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Parameter_Association);
      return Node3 (N);
   end Actual_Parameter;

   function Aliased_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Constrained_Array_Definition
          or else NT (N).Nkind = N_Object_Declaration
          or else NT (N).Nkind = N_Unconstrained_Array_Definition);
      return Flag1 (N);
   end Aliased_Present;

   function All_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Access_To_Object_Definition);
      return Flag1 (N);
   end All_Present;

   function Alternatives
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Case_Statement);
      return List4 (N);
   end Alternatives;

   function Analyzed
      (N : Node_Id) return Boolean is
   begin
      return Flag12 (N);
   end Analyzed;

   function Array_Aggregate
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Enumeration_Representation_Clause);
      return Node4 (N);
   end Array_Aggregate;

   function Assignment_OK
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      return Flag1 (N);
   end Assignment_OK;

   function Attribute_Name
      (N : Node_Id) return Name_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Attribute_Reference);
      return Name2 (N);
   end Attribute_Name;

   function Bad_Is_Detected
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Subprogram_Body);
      return Flag1 (N);
   end Bad_Is_Detected;

   function Body_Required
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      return Flag3 (N);
   end Body_Required;

   function Box_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Package_Declaration
          or else NT (N).Nkind = N_Formal_Subprogram_Declaration);
      return Flag1 (N);
   end Box_Present;

   function Char_Literal_Value
      (N : Node_Id) return Char_Code is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Character_Literal);
      return Char_Code2 (N);
   end Char_Literal_Value;

   function Chars
      (N : Node_Id) return Name_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Has_Chars);
      return Name1 (N);
   end Chars;

   function Choice_Parameter
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Exception_Handler);
      return Node2 (N);
   end Choice_Parameter;

   function Choices
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Association);
      return List1 (N);
   end Choices;

   function Comes_From_Source
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Character_Literal
          or else NT (N).Nkind = N_Defining_Character_Literal
          or else NT (N).Nkind = N_Defining_Identifier
          or else NT (N).Nkind = N_Defining_Operator_Symbol
          or else NT (N).Nkind = N_Identifier
          or else NT (N).Nkind = N_Integer_Literal
          or else NT (N).Nkind = N_Operator_Symbol
          or else NT (N).Nkind = N_Real_Literal
          or else NT (N).Nkind = N_String_Literal);
      return Flag2 (N);
   end Comes_From_Source;

   function Component_Associations
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Aggregate
          or else NT (N).Nkind = N_Extension_Aggregate);
      return List4 (N);
   end Component_Associations;

   function Component_Clauses
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Record_Representation_Clause);
      return List3 (N);
   end Component_Clauses;

   function Component_Declarations
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_List);
      return List3 (N);
   end Component_Declarations;

   function Component_List
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Record_Definition
          or else NT (N).Nkind = N_Variant);
      return Node1 (N);
   end Component_List;

   function Component_Name
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Clause);
      return Node1 (N);
   end Component_Name;

   function Condition
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Alternative
          or else NT (N).Nkind = N_Delay_Alternative
          or else NT (N).Nkind = N_Elsif_Part
          or else NT (N).Nkind = N_Entry_Body_Formal_Part
          or else NT (N).Nkind = N_Exit_Statement
          or else NT (N).Nkind = N_If_Statement
          or else NT (N).Nkind = N_Iteration_Scheme
          or else NT (N).Nkind = N_Terminate_Alternative);
      return Node5 (N);
   end Condition;

   function Constant_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Access_To_Object_Definition
          or else NT (N).Nkind = N_Object_Declaration);
      return Flag2 (N);
   end Constant_Present;

   function Constraint
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Subtype_Indication);
      return Node3 (N);
   end Constraint;

   function Constraints
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Index_Or_Discriminant_Constraint);
      return List1 (N);
   end Constraints;

   function Context_Installed
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      return Flag3 (N);
   end Context_Installed;

   function Context_Items
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      return List1 (N);
   end Context_Items;

   function Controlling_Argument
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Procedure_Call_Statement);
      return Node1 (N);
   end Controlling_Argument;

   function Corresponding_Body
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Generic_Package_Declaration
          or else NT (N).Nkind = N_Generic_Subprogram_Declaration
          or else NT (N).Nkind = N_Package_Body_Stub
          or else NT (N).Nkind = N_Package_Declaration
          or else NT (N).Nkind = N_Protected_Body_Stub
          or else NT (N).Nkind = N_Subprogram_Body_Stub
          or else NT (N).Nkind = N_Subprogram_Declaration
          or else NT (N).Nkind = N_Task_Body_Stub);
      return Node5 (N);
   end Corresponding_Body;

   function Corresponding_Spec
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Package_Body
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Subprogram_Renaming_Declaration
          or else NT (N).Nkind = N_Task_Body
          or else NT (N).Nkind = N_With_Clause);
      return Node5 (N);
   end Corresponding_Spec;

   function Debug_Statement
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Pragma);
      return Node3 (N);
   end Debug_Statement;

   function Decimal
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Real_Literal);
      return Flag3 (N);
   end Decimal;

   function Declarations
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Entry_Body
          or else NT (N).Nkind = N_Package_Body
          or else NT (N).Nkind = N_Protected_Body
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Task_Body);
      return List3 (N);
   end Declarations;

   function Default_Name
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Subprogram_Declaration);
      return Node2 (N);
   end Default_Name;

   function Defining_Identifier
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Defining_Program_Unit_Name
          or else NT (N).Nkind = N_Discriminant_Specification
          or else NT (N).Nkind = N_Entry_Body
          or else NT (N).Nkind = N_Entry_Declaration
          or else NT (N).Nkind = N_Entry_Index_Specification
          or else NT (N).Nkind = N_Exception_Declaration
          or else NT (N).Nkind = N_Exception_Renaming_Declaration
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Formal_Package_Declaration
          or else NT (N).Nkind = N_Formal_Type_Declaration
          or else NT (N).Nkind = N_Full_Type_Declaration
          or else NT (N).Nkind = N_Implicit_Label_Declaration
          or else NT (N).Nkind = N_Incomplete_Type_Declaration
          or else NT (N).Nkind = N_Loop_Parameter_Specification
          or else NT (N).Nkind = N_Number_Declaration
          or else NT (N).Nkind = N_Object_Declaration
          or else NT (N).Nkind = N_Object_Renaming_Declaration
          or else NT (N).Nkind = N_Package_Body_Stub
          or else NT (N).Nkind = N_Parameter_Specification
          or else NT (N).Nkind = N_Private_Extension_Declaration
          or else NT (N).Nkind = N_Private_Type_Declaration
          or else NT (N).Nkind = N_Protected_Body
          or else NT (N).Nkind = N_Protected_Body_Stub
          or else NT (N).Nkind = N_Protected_Type_Declaration
          or else NT (N).Nkind = N_Single_Protected_Declaration
          or else NT (N).Nkind = N_Single_Task_Declaration
          or else NT (N).Nkind = N_Subtype_Declaration
          or else NT (N).Nkind = N_Task_Body
          or else NT (N).Nkind = N_Task_Body_Stub
          or else NT (N).Nkind = N_Task_Type_Declaration);
      return Node1 (N);
   end Defining_Identifier;

   function Defining_Unit_Name
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Function_Instantiation
          or else NT (N).Nkind = N_Function_Specification
          or else NT (N).Nkind = N_Generic_Function_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Procedure_Renaming_Declaration
          or else NT (N).Nkind = N_Package_Body
          or else NT (N).Nkind = N_Package_Instantiation
          or else NT (N).Nkind = N_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Package_Specification
          or else NT (N).Nkind = N_Procedure_Instantiation
          or else NT (N).Nkind = N_Procedure_Specification);
      return Node1 (N);
   end Defining_Unit_Name;

   function Delay_Alternative
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Timed_Entry_Call);
      return Node4 (N);
   end Delay_Alternative;

   function Delay_Statement
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Delay_Alternative);
      return Node2 (N);
   end Delay_Statement;

   function Delta_Expression
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Decimal_Fixed_Point_Definition
          or else NT (N).Nkind = N_Delta_Constraint
          or else NT (N).Nkind = N_Ordinary_Fixed_Point_Definition);
      return Node3 (N);
   end Delta_Expression;

   function Denominator
      (N : Node_Id) return Uint is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Real_Literal);
      return Uint4 (N);
   end Denominator;

   function Digits_Expression
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Decimal_Fixed_Point_Definition
          or else NT (N).Nkind = N_Digits_Constraint
          or else NT (N).Nkind = N_Floating_Point_Definition);
      return Node2 (N);
   end Digits_Expression;

   function Discrete_Choices
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Case_Statement_Alternative
          or else NT (N).Nkind = N_Variant);
      return List4 (N);
   end Discrete_Choices;

   function Discrete_Range
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Slice);
      return Node4 (N);
   end Discrete_Range;

   function Discrete_Subtype_Definition
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Declaration
          or else NT (N).Nkind = N_Entry_Index_Specification
          or else NT (N).Nkind = N_Loop_Parameter_Specification);
      return Node4 (N);
   end Discrete_Subtype_Definition;

   function Discrete_Subtype_Definitions
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Constrained_Array_Definition);
      return List2 (N);
   end Discrete_Subtype_Definitions;

   function Discriminant_Specifications
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Type_Declaration
          or else NT (N).Nkind = N_Full_Type_Declaration
          or else NT (N).Nkind = N_Incomplete_Type_Declaration
          or else NT (N).Nkind = N_Private_Extension_Declaration
          or else NT (N).Nkind = N_Private_Type_Declaration
          or else NT (N).Nkind = N_Protected_Type_Declaration
          or else NT (N).Nkind = N_Task_Type_Declaration);
      return List4 (N);
   end Discriminant_Specifications;

   function Discriminant_Type
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Discriminant_Specification);
      return Node2 (N);
   end Discriminant_Type;

   function Do_Access_Check
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Attribute_Reference
          or else NT (N).Nkind = N_Explicit_Dereference
          or else NT (N).Nkind = N_Indexed_Component
          or else NT (N).Nkind = N_Selected_Component
          or else NT (N).Nkind = N_Slice);
      return Flag2 (N);
   end Do_Access_Check;

   function Do_Accessibility_Check
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Parameter_Specification);
      return Flag3 (N);
   end Do_Accessibility_Check;

   function Do_Discriminant_Check
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Selected_Component);
      return Flag3 (N);
   end Do_Discriminant_Check;

   function Do_Division_Check
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Op_Divide
          or else NT (N).Nkind = N_Op_Mod
          or else NT (N).Nkind = N_Op_Rem);
      return Flag3 (N);
   end Do_Division_Check;

   function Do_Elaboration_Check
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Call_Statement
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Procedure_Call_Statement);
      return Flag2 (N);
   end Do_Elaboration_Check;

   function Do_Length_Check
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Assignment_Statement
          or else NT (N).Nkind = N_Op_And
          or else NT (N).Nkind = N_Op_Or
          or else NT (N).Nkind = N_Op_Xor
          or else NT (N).Nkind = N_Type_Conversion);
      return Flag4 (N);
   end Do_Length_Check;

   function Do_Overflow_Check
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Op
          or else NT (N).Nkind = N_Type_Conversion);
      return Flag2 (N);
   end Do_Overflow_Check;

   function Do_Range_Check
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      return Flag9 (N);
   end Do_Range_Check;

   function Do_Storage_Check
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Allocator
          or else NT (N).Nkind = N_Subprogram_Body);
      return Flag2 (N);
   end Do_Storage_Check;

   function Do_Tag_Check
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Assignment_Statement
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Procedure_Call_Statement
          or else NT (N).Nkind = N_Return_Statement
          or else NT (N).Nkind = N_Type_Conversion);
      return Flag3 (N);
   end Do_Tag_Check;

   function Elaborate_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      return Flag4 (N);
   end Elaborate_Present;

   function Elaborate_All_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      return Flag1 (N);
   end Elaborate_All_Present;

   function Elaborate_Body_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      return Flag7 (N);
   end Elaborate_Body_Present;

   function Else_Statements
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Conditional_Entry_Call
          or else NT (N).Nkind = N_If_Statement
          or else NT (N).Nkind = N_Selective_Accept);
      return List4 (N);
   end Else_Statements;

   function Elsif_Parts
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_If_Statement);
      return List3 (N);
   end Elsif_Parts;

   function Enclosing_Variant
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Variant);
      return Node2 (N);
   end Enclosing_Variant;

   function Entity
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Op
          or else NT (N).Nkind = N_Attribute_Reference
          or else NT (N).Nkind = N_Character_Literal
          or else NT (N).Nkind = N_Expanded_Name
          or else NT (N).Nkind = N_Freeze_Entity
          or else NT (N).Nkind = N_Identifier
          or else NT (N).Nkind = N_Operator_Symbol);
      return Node4 (N);
   end Entity;

   function Entry_Body_Formal_Part
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Body);
      return Node5 (N);
   end Entry_Body_Formal_Part;

   function Entry_Call_Alternative
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Conditional_Entry_Call
          or else NT (N).Nkind = N_Timed_Entry_Call);
      return Node1 (N);
   end Entry_Call_Alternative;

   function Entry_Call_Statement
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Call_Alternative);
      return Node1 (N);
   end Entry_Call_Statement;

   function Entry_Direct_Name
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Statement);
      return Node1 (N);
   end Entry_Direct_Name;

   function Entry_Index
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Statement);
      return Node5 (N);
   end Entry_Index;

   function Entry_Index_Specification
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Body_Formal_Part);
      return Node1 (N);
   end Entry_Index_Specification;

   function Error_Posted
      (N : Node_Id) return Boolean is
   begin
      return Flag13 (N);
   end Error_Posted;

   function Etype
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
         or else NT (N).Nkind in N_Has_Etype);
      return Node5 (N);
   end Etype;

   function Evaluate_Once
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      return Flag10 (N);
   end Evaluate_Once;

   function Exception_Choices
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Exception_Handler);
      return List4 (N);
   end Exception_Choices;

   function Exception_Handlers
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Handled_Sequence_Of_Statements);
      return List4 (N);
   end Exception_Handlers;

   function Explicit_Generic_Actual_Parameter
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Generic_Association);
      return Node1 (N);
   end Explicit_Generic_Actual_Parameter;

   function Expression
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Allocator
          or else NT (N).Nkind = N_Assignment_Statement
          or else NT (N).Nkind = N_At_Clause
          or else NT (N).Nkind = N_Attribute_Definition_Clause
          or else NT (N).Nkind = N_Case_Statement
          or else NT (N).Nkind = N_Code_Statement
          or else NT (N).Nkind = N_Component_Association
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Delay_Relative_Statement
          or else NT (N).Nkind = N_Delay_Until_Statement
          or else NT (N).Nkind = N_Discriminant_Association
          or else NT (N).Nkind = N_Discriminant_Specification
          or else NT (N).Nkind = N_Expression_Actions
          or else NT (N).Nkind = N_Extension_Aggregate
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Free_Statement
          or else NT (N).Nkind = N_Mod_Clause
          or else NT (N).Nkind = N_Modular_Type_Definition
          or else NT (N).Nkind = N_Number_Declaration
          or else NT (N).Nkind = N_Object_Declaration
          or else NT (N).Nkind = N_Parameter_Specification
          or else NT (N).Nkind = N_Pragma_Argument_Association
          or else NT (N).Nkind = N_Qualified_Expression
          or else NT (N).Nkind = N_Return_Statement
          or else NT (N).Nkind = N_Type_Conversion
          or else NT (N).Nkind = N_Unchecked_Type_Conversion);
      return Node3 (N);
   end Expression;

   function Expressions
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Aggregate
          or else NT (N).Nkind = N_Attribute_Reference
          or else NT (N).Nkind = N_Concat_Multiple
          or else NT (N).Nkind = N_Conditional_Expression
          or else NT (N).Nkind = N_Indexed_Component);
      return List1 (N);
   end Expressions;

   function First_Bit
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Clause);
      return Node3 (N);
   end First_Bit;

   function First_Itype
      (N : Node_Id) return Entity_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Has_Itypes);
      return Node2 (N);
   end First_Itype;

   function First_Name
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      return Flag5 (N);
   end First_Name;

   function First_Named_Actual
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Call_Statement
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Procedure_Call_Statement);
      return Node4 (N);
   end First_Named_Actual;

   function Following_Pragmas
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      return List3 (N);
   end Following_Pragmas;

   function Formal_Type_Definition
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Type_Declaration);
      return Node3 (N);
   end Formal_Type_Definition;

   function Generic_Associations
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Package_Declaration
          or else NT (N).Nkind = N_Function_Instantiation
          or else NT (N).Nkind = N_Package_Instantiation
          or else NT (N).Nkind = N_Procedure_Instantiation);
      return List3 (N);
   end Generic_Associations;

   function Generic_Formal_Declarations
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Generic_Package_Declaration
          or else NT (N).Nkind = N_Generic_Subprogram_Declaration);
      return List3 (N);
   end Generic_Formal_Declarations;

   function Generic_Parent
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Function_Specification
          or else NT (N).Nkind = N_Package_Specification
          or else NT (N).Nkind = N_Procedure_Specification);
      return Node5 (N);
   end Generic_Parent;

   function Handled_Statement_Sequence
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Statement
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Entry_Body
          or else NT (N).Nkind = N_Package_Body
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Task_Body);
      return Node4 (N);
   end Handled_Statement_Sequence;

   function Has_Created_Identifier
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Loop_Statement);
      return Flag1 (N);
   end Has_Created_Identifier;

   function Has_Dynamic_Itype
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Has_Itypes);
      return Flag14 (N);
   end Has_Dynamic_Itype;

   function Has_No_Side_Effects
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      return Flag8 (N);
   end Has_No_Side_Effects;

   function Has_Priority_Pragma
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Protected_Definition
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Task_Definition);
      return Flag6 (N);
   end Has_Priority_Pragma;

   function Has_Private_View
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
         or else NT (N).Nkind in N_Op
         or else NT (N).Nkind = N_Character_Literal
         or else NT (N).Nkind = N_Expanded_Name
         or else NT (N).Nkind = N_Identifier
         or else NT (N).Nkind = N_Operator_Symbol);
      return Flag11 (N);
   end Has_Private_View;

   function Has_Storage_Size_Pragma
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Task_Definition);
      return Flag5 (N);
   end Has_Storage_Size_Pragma;

   function Has_Unknown_Discriminants
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Type_Declaration
          or else NT (N).Nkind = N_Incomplete_Type_Declaration
          or else NT (N).Nkind = N_Private_Extension_Declaration
          or else NT (N).Nkind = N_Private_Type_Declaration);
      return Flag3 (N);
   end Has_Unknown_Discriminants;

   function High_Bound
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Range
          or else NT (N).Nkind = N_Real_Range_Specification
          or else NT (N).Nkind = N_Signed_Integer_Type_Definition);
      return Node2 (N);
   end High_Bound;

   function Homonym
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Defining_Character_Literal
          or else NT (N).Nkind = N_Defining_Identifier
          or else NT (N).Nkind = N_Defining_Operator_Symbol);
      return Node4 (N);
   end Homonym;

   function Identifier
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_At_Clause
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Designator
          or else NT (N).Nkind = N_Enumeration_Representation_Clause
          or else NT (N).Nkind = N_Handled_Sequence_Of_Statements
          or else NT (N).Nkind = N_Label
          or else NT (N).Nkind = N_Loop_Statement
          or else NT (N).Nkind = N_Record_Representation_Clause);
      return Node1 (N);
   end Identifier;

   function Implicit_With
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      return Flag2 (N);
   end Implicit_With;

   function In_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Parameter_Specification);
      return Flag1 (N);
   end In_Present;

   function Intval
      (N : Node_Id) return Uint is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Integer_Literal);
      return Uint3 (N);
   end Intval;

   function Is_Folded
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      return Flag7 (N);
   end Is_Folded;

   function Is_Overloaded
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      return Flag5 (N);
   end Is_Overloaded;

   function Is_Static
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      return Flag6 (N);
   end Is_Static;

   function Is_Task_Master
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Task_Body);
      return Flag5 (N);
   end Is_Task_Master;

   function Iteration_Scheme
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Loop_Statement);
      return Node2 (N);
   end Iteration_Scheme;

   function Label
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Implicit_Label_Declaration);
      return Node2 (N);
   end Label;

   function Last_Bit
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Clause);
      return Node4 (N);
   end Last_Bit;

   function Last_Name
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      return Flag6 (N);
   end Last_Name;

   function Library_Unit
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit
          or else NT (N).Nkind = N_Package_Body_Stub
          or else NT (N).Nkind = N_Protected_Body_Stub
          or else NT (N).Nkind = N_Subprogram_Body_Stub
          or else NT (N).Nkind = N_Task_Body_Stub
          or else NT (N).Nkind = N_With_Clause);
      return Node4 (N);
   end Library_Unit;

   function Left_Opnd
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Binary_Op);
      return Node2 (N);
   end Left_Opnd;

   function Literals
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Enumeration_Type_Definition);
      return List1 (N);
   end Literals;

   function Limited_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Private_Type_Definition
          or else NT (N).Nkind = N_Private_Type_Declaration
          or else NT (N).Nkind = N_Record_Definition);
      return Flag2 (N);
   end Limited_Present;

   function Loop_Parameter_Specification
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Iteration_Scheme);
      return Node1 (N);
   end Loop_Parameter_Specification;

   function Low_Bound
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Range
          or else NT (N).Nkind = N_Real_Range_Specification
          or else NT (N).Nkind = N_Signed_Integer_Type_Definition);
      return Node1 (N);
   end Low_Bound;

   function Mod_Clause
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Record_Representation_Clause);
      return Node2 (N);
   end Mod_Clause;

   function More_Ids
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Discriminant_Specification
          or else NT (N).Nkind = N_Exception_Declaration
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Number_Declaration
          or else NT (N).Nkind = N_Object_Declaration
          or else NT (N).Nkind = N_Parameter_Specification);
      return Flag5 (N);
   end More_Ids;

   function Name
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Assignment_Statement
          or else NT (N).Nkind = N_Attribute_Definition_Clause
          or else NT (N).Nkind = N_Defining_Program_Unit_Name
          or else NT (N).Nkind = N_Designator
          or else NT (N).Nkind = N_Entry_Call_Statement
          or else NT (N).Nkind = N_Exception_Renaming_Declaration
          or else NT (N).Nkind = N_Exit_Statement
          or else NT (N).Nkind = N_Formal_Package_Declaration
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Function_Instantiation
          or else NT (N).Nkind = N_Generic_Function_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Procedure_Renaming_Declaration
          or else NT (N).Nkind = N_Goto_Statement
          or else NT (N).Nkind = N_Object_Renaming_Declaration
          or else NT (N).Nkind = N_Package_Instantiation
          or else NT (N).Nkind = N_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Procedure_Call_Statement
          or else NT (N).Nkind = N_Procedure_Instantiation
          or else NT (N).Nkind = N_Raise_Statement
          or else NT (N).Nkind = N_Requeue_Statement
          or else NT (N).Nkind = N_Subprogram_Renaming_Declaration
          or else NT (N).Nkind = N_Subunit
          or else NT (N).Nkind = N_Variant_Part
          or else NT (N).Nkind = N_With_Clause);
      return Node2 (N);
   end Name;

   function Names
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Abort_Statement
          or else NT (N).Nkind = N_Use_Package_Clause);
      return List2 (N);
   end Names;

   function Next_Entity
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Defining_Character_Literal
          or else NT (N).Nkind = N_Defining_Identifier
          or else NT (N).Nkind = N_Defining_Operator_Symbol);
      return Node2 (N);
   end Next_Entity;

   function Next_Named_Actual
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Parameter_Association);
      return Node4 (N);
   end Next_Named_Actual;

   function No_Defer
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Raise_Statement);
      return Flag1 (N);
   end No_Defer;

   function Null_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_List
          or else NT (N).Nkind = N_Record_Definition);
      return Flag3 (N);
   end Null_Present;

   function Null_Record_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Aggregate
          or else NT (N).Nkind = N_Extension_Aggregate);
      return Flag2 (N);
   end Null_Record_Present;

   function Numerator
      (N : Node_Id) return Uint is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Real_Literal);
      return Uint3 (N);
   end Numerator;

   function Object_Definition
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Object_Declaration);
      return Node4 (N);
   end Object_Definition;

   function Others_Discrete_Choices
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Others_Choice);
      return List1 (N);
   end Others_Discrete_Choices;

   function Out_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Parameter_Specification);
      return Flag2 (N);
   end Out_Present;

   function Parameter_Associations
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Call_Statement
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Procedure_Call_Statement);
      return List3 (N);
   end Parameter_Associations;

   function Parameter_Specifications
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Statement
          or else NT (N).Nkind = N_Access_Function_Definition
          or else NT (N).Nkind = N_Access_Procedure_Definition
          or else NT (N).Nkind = N_Entry_Body_Formal_Part
          or else NT (N).Nkind = N_Entry_Declaration
          or else NT (N).Nkind = N_Function_Specification
          or else NT (N).Nkind = N_Procedure_Specification);
      return List3 (N);
   end Parameter_Specifications;

   function Parameter_Type
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Parameter_Specification);
      return Node2 (N);
   end Parameter_Type;

   function Parent_Spec
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Function_Instantiation
          or else NT (N).Nkind = N_Generic_Function_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Package_Declaration
          or else NT (N).Nkind = N_Generic_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Procedure_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Subprogram_Declaration
          or else NT (N).Nkind = N_Package_Declaration
          or else NT (N).Nkind = N_Package_Instantiation
          or else NT (N).Nkind = N_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Procedure_Instantiation
          or else NT (N).Nkind = N_Subprogram_Declaration
          or else NT (N).Nkind = N_Subprogram_Renaming_Declaration);
      return Node4 (N);
   end Parent_Spec;

   function Position
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Clause);
      return Node2 (N);
   end Position;

   function Pragma_Argument_Associations
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Pragma);
      return List2 (N);
   end Pragma_Argument_Associations;

   function Preelaborable
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      return Flag2 (N);
   end Preelaborable;

   function Prefix
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Attribute_Reference
          or else NT (N).Nkind = N_Expanded_Name
          or else NT (N).Nkind = N_Explicit_Dereference
          or else NT (N).Nkind = N_Indexed_Component
          or else NT (N).Nkind = N_Reference
          or else NT (N).Nkind = N_Selected_Component
          or else NT (N).Nkind = N_Slice);
      return Node3 (N);
   end Prefix;

   function Prev_Ids
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Discriminant_Specification
          or else NT (N).Nkind = N_Exception_Declaration
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Number_Declaration
          or else NT (N).Nkind = N_Object_Declaration
          or else NT (N).Nkind = N_Parameter_Specification);
      return Flag6 (N);
   end Prev_Ids;

   function Private_Declarations
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Package_Specification
          or else NT (N).Nkind = N_Protected_Definition
          or else NT (N).Nkind = N_Task_Definition);
      return List4 (N);
   end Private_Declarations;

   function Private_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit
          or else NT (N).Nkind = N_Formal_Derived_Type_Definition);
      return Flag1 (N);
   end Private_Present;

   function Procedure_To_Call
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Allocator
          or else NT (N).Nkind = N_Free_Statement);
      return Node4 (N);
   end Procedure_To_Call;

   function Proper_Body
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Subunit);
      return Node1 (N);
   end Proper_Body;

   function Protected_Definition
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Protected_Type_Declaration
          or else NT (N).Nkind = N_Single_Protected_Declaration);
      return Node3 (N);
   end Protected_Definition;

   function Protected_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Access_Function_Definition
          or else NT (N).Nkind = N_Access_Procedure_Definition);
      return Flag1 (N);
   end Protected_Present;

   function Range_Constraint
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Delta_Constraint
          or else NT (N).Nkind = N_Digits_Constraint);
      return Node4 (N);
   end Range_Constraint;

   function Range_Expression
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Range_Constraint);
      return Node4 (N);
   end Range_Expression;

   function Real_Range_Specification
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Decimal_Fixed_Point_Definition
          or else NT (N).Nkind = N_Floating_Point_Definition
          or else NT (N).Nkind = N_Ordinary_Fixed_Point_Definition);
      return Node4 (N);
   end Real_Range_Specification;

   function Record_Extension_Part
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Derived_Type_Definition);
      return Node3 (N);
   end Record_Extension_Part;

   function Redundant_Use
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Expanded_Name
          or else NT (N).Nkind = N_Identifier);
      return Flag3 (N);
   end Redundant_Use;

   function Return_Type
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Return_Statement);
      return Node1 (N);
   end Return_Type;

   function Reverse_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Loop_Parameter_Specification);
      return Flag1 (N);
   end Reverse_Present;

   function Right_Opnd
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Op);
      return Node3 (N);
   end Right_Opnd;

   function Scope
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Defining_Character_Literal
          or else NT (N).Nkind = N_Defining_Identifier
          or else NT (N).Nkind = N_Defining_Operator_Symbol);
      return Node3 (N);
   end Scope;

   function Selective_Accept_Alternatives
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Selective_Accept);
      return List1 (N);
   end Selective_Accept_Alternatives;

   function Selector_Name
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Expanded_Name
          or else NT (N).Nkind = N_Generic_Association
          or else NT (N).Nkind = N_Parameter_Association
          or else NT (N).Nkind = N_Selected_Component);
      return Node2 (N);
   end Selector_Name;

   function Selector_Names
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Discriminant_Association);
      return List1 (N);
   end Selector_Names;

   function Specification
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Abstract_Subprogram_Declaration
          or else NT (N).Nkind = N_Formal_Subprogram_Declaration
          or else NT (N).Nkind = N_Generic_Package_Declaration
          or else NT (N).Nkind = N_Generic_Subprogram_Declaration
          or else NT (N).Nkind = N_Package_Declaration
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Subprogram_Body_Stub
          or else NT (N).Nkind = N_Subprogram_Declaration
          or else NT (N).Nkind = N_Subprogram_Renaming_Declaration);
      return Node1 (N);
   end Specification;

   function Statements
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Abortable_Part
          or else NT (N).Nkind = N_Accept_Alternative
          or else NT (N).Nkind = N_Case_Statement_Alternative
          or else NT (N).Nkind = N_Delay_Alternative
          or else NT (N).Nkind = N_Entry_Call_Alternative
          or else NT (N).Nkind = N_Exception_Handler
          or else NT (N).Nkind = N_Handled_Sequence_Of_Statements
          or else NT (N).Nkind = N_Loop_Statement
          or else NT (N).Nkind = N_Triggering_Alternative);
      return List3 (N);
   end Statements;

   function Storage_Pool
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Allocator
          or else NT (N).Nkind = N_Free_Statement);
      return Node1 (N);
   end Storage_Pool;

   function Strval
      (N : Node_Id) return String_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Operator_Symbol
          or else NT (N).Nkind = N_String_Literal);
      return Str3 (N);
   end Strval;

   function Subtype_Indication
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Access_To_Object_Definition
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Constrained_Array_Definition
          or else NT (N).Nkind = N_Derived_Type_Definition
          or else NT (N).Nkind = N_Private_Extension_Declaration
          or else NT (N).Nkind = N_Subtype_Declaration
          or else NT (N).Nkind = N_Unconstrained_Array_Definition);
      return Node5 (N);
   end Subtype_Indication;

   function Subtype_Mark
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Access_Definition
          or else NT (N).Nkind = N_Access_Function_Definition
          or else NT (N).Nkind = N_Formal_Derived_Type_Definition
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Function_Specification
          or else NT (N).Nkind = N_Object_Renaming_Declaration
          or else NT (N).Nkind = N_Qualified_Expression
          or else NT (N).Nkind = N_Subtype_Indication
          or else NT (N).Nkind = N_Type_Conversion
          or else NT (N).Nkind = N_Unchecked_Type_Conversion);
      return Node4 (N);
   end Subtype_Mark;

   function Subtype_Marks
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Unconstrained_Array_Definition
          or else NT (N).Nkind = N_Use_Type_Clause);
      return List2 (N);
   end Subtype_Marks;

   function Tagged_Present
      (N : Node_Id) return Boolean is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Private_Type_Definition
          or else NT (N).Nkind = N_Private_Type_Declaration
          or else NT (N).Nkind = N_Record_Definition);
      return Flag1 (N);
   end Tagged_Present;

   function Task_Definition
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Single_Task_Declaration
          or else NT (N).Nkind = N_Task_Type_Declaration);
      return Node3 (N);
   end Task_Definition;

   function Then_Statements
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Elsif_Part
          or else NT (N).Nkind = N_If_Statement);
      return List2 (N);
   end Then_Statements;

   function Triggering_Alternative
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Asynchronous_Select);
      return Node1 (N);
   end Triggering_Alternative;

   function Triggering_Statement
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Triggering_Alternative);
      return Node1 (N);
   end Triggering_Statement;

   function Type_Definition
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Full_Type_Declaration);
      return Node3 (N);
   end Type_Definition;

   function Unit
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      return Node2 (N);
   end Unit;

   function Variant_Part
      (N : Node_Id) return Node_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_List);
      return Node4 (N);
   end Variant_Part;

   function Variants
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Variant_Part);
      return List1 (N);
   end Variants;

   function Visible_Declarations
      (N : Node_Id) return List_Id is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Package_Specification
          or else NT (N).Nkind = N_Protected_Definition
          or else NT (N).Nkind = N_Task_Definition);
      return List2 (N);
   end Visible_Declarations;

   --------------------------
   -- Field Set Procedures --
   --------------------------

   procedure Set_Abort_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Requeue_Statement);
      Set_Flag1 (N, Val);
   end Set_Abort_Present;

   procedure Set_Abortable_Part
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Asynchronous_Select);
      Set_Node2_With_Parent (N, Val);
   end Set_Abortable_Part;

   procedure Set_Abstract_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Derived_Type_Definition
          or else NT (N).Nkind = N_Formal_Private_Type_Definition
          or else NT (N).Nkind = N_Private_Extension_Declaration
          or else NT (N).Nkind = N_Private_Type_Declaration
          or else NT (N).Nkind = N_Record_Definition);
      Set_Flag4 (N, Val);
   end Set_Abstract_Present;

   procedure Set_Accept_Statement
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Alternative);
      Set_Node2_With_Parent (N, Val);
   end Set_Accept_Statement;

   procedure Set_Actions
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Expression_Actions);
      Set_List1_With_Parent (N, Val);
   end Set_Actions;

   procedure Set_Activation_Chain_Entity
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Generic_Package_Declaration
          or else NT (N).Nkind = N_Package_Declaration
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Task_Body);
      Set_Node2 (N, Val); -- semantic node, no parent set
   end Set_Activation_Chain_Entity;

   procedure Set_Acts_As_Spec
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit
          or else NT (N).Nkind = N_Subprogram_Body);
      Set_Flag4 (N, Val);
   end Set_Acts_As_Spec;

   procedure Set_Actual_Parameter
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Parameter_Association);
      Set_Node3_With_Parent (N, Val);
   end Set_Actual_Parameter;

   procedure Set_Aliased_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Constrained_Array_Definition
          or else NT (N).Nkind = N_Object_Declaration
          or else NT (N).Nkind = N_Unconstrained_Array_Definition);
      Set_Flag1 (N, Val);
   end Set_Aliased_Present;

   procedure Set_All_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Access_To_Object_Definition);
      Set_Flag1 (N, Val);
   end Set_All_Present;

   procedure Set_Alternatives
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Case_Statement);
      Set_List4_With_Parent (N, Val);
   end Set_Alternatives;

   procedure Set_Analyzed
      (N : Node_Id; Val : Boolean := True) is
   begin
      Set_Flag12 (N, Val);
   end Set_Analyzed;

   procedure Set_Array_Aggregate
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Enumeration_Representation_Clause);
      Set_Node4_With_Parent (N, Val);
   end Set_Array_Aggregate;

   procedure Set_Assignment_OK
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      Set_Flag1 (N, Val);
   end Set_Assignment_OK;

   procedure Set_Attribute_Name
      (N : Node_Id; Val : Name_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Attribute_Reference);
      Set_Name2 (N, Val);
   end Set_Attribute_Name;

   procedure Set_Bad_Is_Detected
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Subprogram_Body);
      Set_Flag1 (N, Val);
   end Set_Bad_Is_Detected;

   procedure Set_Body_Required
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      Set_Flag3 (N, Val);
   end Set_Body_Required;

   procedure Set_Box_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Package_Declaration
          or else NT (N).Nkind = N_Formal_Subprogram_Declaration);
      Set_Flag1 (N, Val);
   end Set_Box_Present;

   procedure Set_Char_Literal_Value
      (N : Node_Id; Val : Char_Code) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Character_Literal);
      Set_Char_Code2 (N, Val);
   end Set_Char_Literal_Value;

   procedure Set_Chars
      (N : Node_Id; Val : Name_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Has_Chars);
      Set_Name1 (N, Val);
   end Set_Chars;

   procedure Set_Choice_Parameter
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Exception_Handler);
      Set_Node2_With_Parent (N, Val);
   end Set_Choice_Parameter;

   procedure Set_Choices
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Association);
      Set_List1_With_Parent (N, Val);
   end Set_Choices;

   procedure Set_Comes_From_Source
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Character_Literal
          or else NT (N).Nkind = N_Defining_Character_Literal
          or else NT (N).Nkind = N_Defining_Identifier
          or else NT (N).Nkind = N_Defining_Operator_Symbol
          or else NT (N).Nkind = N_Identifier
          or else NT (N).Nkind = N_Integer_Literal
          or else NT (N).Nkind = N_Operator_Symbol
          or else NT (N).Nkind = N_Real_Literal
          or else NT (N).Nkind = N_String_Literal);
      Set_Flag2 (N, Val);
   end Set_Comes_From_Source;

   procedure Set_Component_Associations
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Aggregate
          or else NT (N).Nkind = N_Extension_Aggregate);
      Set_List4_With_Parent (N, Val);
   end Set_Component_Associations;

   procedure Set_Component_Clauses
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Record_Representation_Clause);
      Set_List3_With_Parent (N, Val);
   end Set_Component_Clauses;

   procedure Set_Component_Declarations
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_List);
      Set_List3_With_Parent (N, Val);
   end Set_Component_Declarations;

   procedure Set_Component_List
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Record_Definition
          or else NT (N).Nkind = N_Variant);
      Set_Node1_With_Parent (N, Val);
   end Set_Component_List;

   procedure Set_Component_Name
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Clause);
      Set_Node1_With_Parent (N, Val);
   end Set_Component_Name;

   procedure Set_Condition
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Alternative
          or else NT (N).Nkind = N_Delay_Alternative
          or else NT (N).Nkind = N_Elsif_Part
          or else NT (N).Nkind = N_Entry_Body_Formal_Part
          or else NT (N).Nkind = N_Exit_Statement
          or else NT (N).Nkind = N_If_Statement
          or else NT (N).Nkind = N_Iteration_Scheme
          or else NT (N).Nkind = N_Terminate_Alternative);
      Set_Node5_With_Parent (N, Val);
   end Set_Condition;

   procedure Set_Constant_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Access_To_Object_Definition
          or else NT (N).Nkind = N_Object_Declaration);
      Set_Flag2 (N, Val);
   end Set_Constant_Present;

   procedure Set_Constraint
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Subtype_Indication);
      Set_Node3_With_Parent (N, Val);
   end Set_Constraint;

   procedure Set_Constraints
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Index_Or_Discriminant_Constraint);
      Set_List1_With_Parent (N, Val);
   end Set_Constraints;

   procedure Set_Context_Installed
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      Set_Flag3 (N, Val);
   end Set_Context_Installed;

   procedure Set_Context_Items
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      Set_List1_With_Parent (N, Val);
   end Set_Context_Items;

   procedure Set_Controlling_Argument
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Procedure_Call_Statement);
      Set_Node1 (N, Val); -- Note: semantic node, no parent set
   end Set_Controlling_Argument;

   procedure Set_Corresponding_Body
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Generic_Package_Declaration
          or else NT (N).Nkind = N_Generic_Subprogram_Declaration
          or else NT (N).Nkind = N_Package_Body_Stub
          or else NT (N).Nkind = N_Package_Declaration
          or else NT (N).Nkind = N_Protected_Body_Stub
          or else NT (N).Nkind = N_Subprogram_Body_Stub
          or else NT (N).Nkind = N_Subprogram_Declaration
          or else NT (N).Nkind = N_Task_Body_Stub);
      Set_Node5 (N, Val); -- Semantic node, no parent set
   end Set_Corresponding_Body;

   procedure Set_Corresponding_Spec
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Package_Body
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Subprogram_Renaming_Declaration
          or else NT (N).Nkind = N_Task_Body
          or else NT (N).Nkind = N_With_Clause);
      Set_Node5 (N, Val); -- Semantic node, no parent set
   end Set_Corresponding_Spec;

   procedure Set_Debug_Statement
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Pragma);
      Set_Node3 (N, Val);  -- semantic node, no parent set
   end Set_Debug_Statement;

   procedure Set_Decimal
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Real_Literal);
      Set_Flag3 (N, Val);
   end Set_Decimal;

   procedure Set_Declarations
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Entry_Body
          or else NT (N).Nkind = N_Package_Body
          or else NT (N).Nkind = N_Protected_Body
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Task_Body);
      Set_List3_With_Parent (N, Val);
   end Set_Declarations;

   procedure Set_Default_Name
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Subprogram_Declaration);
      Set_Node2_With_Parent (N, Val);
   end Set_Default_Name;

   procedure Set_Defining_Identifier
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Defining_Program_Unit_Name
          or else NT (N).Nkind = N_Discriminant_Specification
          or else NT (N).Nkind = N_Entry_Body
          or else NT (N).Nkind = N_Entry_Declaration
          or else NT (N).Nkind = N_Entry_Index_Specification
          or else NT (N).Nkind = N_Exception_Declaration
          or else NT (N).Nkind = N_Exception_Renaming_Declaration
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Formal_Package_Declaration
          or else NT (N).Nkind = N_Formal_Type_Declaration
          or else NT (N).Nkind = N_Full_Type_Declaration
          or else NT (N).Nkind = N_Implicit_Label_Declaration
          or else NT (N).Nkind = N_Incomplete_Type_Declaration
          or else NT (N).Nkind = N_Loop_Parameter_Specification
          or else NT (N).Nkind = N_Number_Declaration
          or else NT (N).Nkind = N_Object_Declaration
          or else NT (N).Nkind = N_Object_Renaming_Declaration
          or else NT (N).Nkind = N_Package_Body_Stub
          or else NT (N).Nkind = N_Parameter_Specification
          or else NT (N).Nkind = N_Private_Extension_Declaration
          or else NT (N).Nkind = N_Private_Type_Declaration
          or else NT (N).Nkind = N_Protected_Body
          or else NT (N).Nkind = N_Protected_Body_Stub
          or else NT (N).Nkind = N_Protected_Type_Declaration
          or else NT (N).Nkind = N_Single_Protected_Declaration
          or else NT (N).Nkind = N_Single_Task_Declaration
          or else NT (N).Nkind = N_Subtype_Declaration
          or else NT (N).Nkind = N_Task_Body
          or else NT (N).Nkind = N_Task_Body_Stub
          or else NT (N).Nkind = N_Task_Type_Declaration);
      Set_Node1_With_Parent (N, Val);
   end Set_Defining_Identifier;

   procedure Set_Defining_Unit_Name
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Function_Instantiation
          or else NT (N).Nkind = N_Function_Specification
          or else NT (N).Nkind = N_Generic_Function_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Procedure_Renaming_Declaration
          or else NT (N).Nkind = N_Package_Body
          or else NT (N).Nkind = N_Package_Instantiation
          or else NT (N).Nkind = N_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Package_Specification
          or else NT (N).Nkind = N_Procedure_Instantiation
          or else NT (N).Nkind = N_Procedure_Specification);
      Set_Node1_With_Parent (N, Val);
   end Set_Defining_Unit_Name;

   procedure Set_Delay_Alternative
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Timed_Entry_Call);
      Set_Node4_With_Parent (N, Val);
   end Set_Delay_Alternative;

   procedure Set_Delay_Statement
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Delay_Alternative);
      Set_Node2_With_Parent (N, Val);
   end Set_Delay_Statement;

   procedure Set_Delta_Expression
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Decimal_Fixed_Point_Definition
          or else NT (N).Nkind = N_Delta_Constraint
          or else NT (N).Nkind = N_Ordinary_Fixed_Point_Definition);
      Set_Node3_With_Parent (N, Val);
   end Set_Delta_Expression;

   procedure Set_Denominator
      (N : Node_Id; Val : Uint) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Real_Literal);
      Set_Uint4 (N, Val);
   end Set_Denominator;

   procedure Set_Digits_Expression
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Decimal_Fixed_Point_Definition
          or else NT (N).Nkind = N_Digits_Constraint
          or else NT (N).Nkind = N_Floating_Point_Definition);
      Set_Node2_With_Parent (N, Val);
   end Set_Digits_Expression;

   procedure Set_Discrete_Choices
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Case_Statement_Alternative
          or else NT (N).Nkind = N_Variant);
      Set_List4_With_Parent (N, Val);
   end Set_Discrete_Choices;

   procedure Set_Discrete_Range
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Slice);
      Set_Node4_With_Parent (N, Val);
   end Set_Discrete_Range;

   procedure Set_Discrete_Subtype_Definition
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Declaration
          or else NT (N).Nkind = N_Entry_Index_Specification
          or else NT (N).Nkind = N_Loop_Parameter_Specification);
      Set_Node4_With_Parent (N, Val);
   end Set_Discrete_Subtype_Definition;

   procedure Set_Discrete_Subtype_Definitions
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Constrained_Array_Definition);
      Set_List2_With_Parent (N, Val);
   end Set_Discrete_Subtype_Definitions;

   procedure Set_Discriminant_Specifications
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Type_Declaration
          or else NT (N).Nkind = N_Full_Type_Declaration
          or else NT (N).Nkind = N_Incomplete_Type_Declaration
          or else NT (N).Nkind = N_Private_Extension_Declaration
          or else NT (N).Nkind = N_Private_Type_Declaration
          or else NT (N).Nkind = N_Protected_Type_Declaration
          or else NT (N).Nkind = N_Task_Type_Declaration);
      Set_List4_With_Parent (N, Val);
   end Set_Discriminant_Specifications;

   procedure Set_Discriminant_Type
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Discriminant_Specification);
      Set_Node2_With_Parent (N, Val);
   end Set_Discriminant_Type;

   procedure Set_Do_Access_Check
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Attribute_Reference
          or else NT (N).Nkind = N_Explicit_Dereference
          or else NT (N).Nkind = N_Indexed_Component
          or else NT (N).Nkind = N_Selected_Component
          or else NT (N).Nkind = N_Slice);
      Set_Flag2 (N, Val);
   end Set_Do_Access_Check;

   procedure Set_Do_Accessibility_Check
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Parameter_Specification);
      Set_Flag3 (N, Val);
   end Set_Do_Accessibility_Check;

   procedure Set_Do_Discriminant_Check
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Selected_Component);
      Set_Flag3 (N, Val);
   end Set_Do_Discriminant_Check;

   procedure Set_Do_Division_Check
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Op_Divide
          or else NT (N).Nkind = N_Op_Mod
          or else NT (N).Nkind = N_Op_Rem);
      Set_Flag3 (N, Val);
   end Set_Do_Division_Check;

   procedure Set_Do_Elaboration_Check
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Call_Statement
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Procedure_Call_Statement);
      Set_Flag2 (N, Val);
   end Set_Do_Elaboration_Check;

   procedure Set_Do_Length_Check
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Assignment_Statement
          or else NT (N).Nkind = N_Op_And
          or else NT (N).Nkind = N_Op_Or
          or else NT (N).Nkind = N_Op_Xor
          or else NT (N).Nkind = N_Type_Conversion);
      Set_Flag4 (N, Val);
   end Set_Do_Length_Check;

   procedure Set_Do_Overflow_Check
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Op
          or else NT (N).Nkind = N_Type_Conversion);
      Set_Flag2 (N, Val);
   end Set_Do_Overflow_Check;

   procedure Set_Do_Range_Check
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      Set_Flag9 (N, Val);
   end Set_Do_Range_Check;

   procedure Set_Do_Storage_Check
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Allocator
          or else NT (N).Nkind = N_Subprogram_Body);
      Set_Flag2 (N, Val);
   end Set_Do_Storage_Check;

   procedure Set_Do_Tag_Check
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Assignment_Statement
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Procedure_Call_Statement
          or else NT (N).Nkind = N_Return_Statement
          or else NT (N).Nkind = N_Type_Conversion);
      Set_Flag3 (N, Val);
   end Set_Do_Tag_Check;

   procedure Set_Elaborate_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      Set_Flag4 (N, Val);
   end Set_Elaborate_Present;

   procedure Set_Elaborate_All_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      Set_Flag1 (N, Val);
   end Set_Elaborate_All_Present;

   procedure Set_Elaborate_Body_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      Set_Flag7 (N, Val);
   end Set_Elaborate_Body_Present;

   procedure Set_Else_Statements
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Conditional_Entry_Call
          or else NT (N).Nkind = N_If_Statement
          or else NT (N).Nkind = N_Selective_Accept);
      Set_List4_With_Parent (N, Val);
   end Set_Else_Statements;

   procedure Set_Elsif_Parts
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_If_Statement);
      Set_List3_With_Parent (N, Val);
   end Set_Elsif_Parts;

   procedure Set_Enclosing_Variant
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Variant);
      Set_Node2 (N, Val); -- semantic field, no parent pointer
   end Set_Enclosing_Variant;

   procedure Set_Entity
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Op
          or else NT (N).Nkind = N_Attribute_Reference
          or else NT (N).Nkind = N_Character_Literal
          or else NT (N).Nkind = N_Expanded_Name
          or else NT (N).Nkind = N_Freeze_Entity
          or else NT (N).Nkind = N_Identifier
          or else NT (N).Nkind = N_Operator_Symbol);
      Set_Node4 (N, Val); -- Note: sem field, don't set parent
   end Set_Entity;

   procedure Set_Entry_Call_Alternative
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Conditional_Entry_Call
          or else NT (N).Nkind = N_Timed_Entry_Call);
      Set_Node1_With_Parent (N, Val);
   end Set_Entry_Call_Alternative;

   procedure Set_Entry_Body_Formal_Part
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Body);
      Set_Node5_With_Parent (N, Val);
   end Set_Entry_Body_Formal_Part;

   procedure Set_Entry_Call_Statement
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Call_Alternative);
      Set_Node1_With_Parent (N, Val);
   end Set_Entry_Call_Statement;

   procedure Set_Entry_Direct_Name
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Statement);
      Set_Node1_With_Parent (N, Val);
   end Set_Entry_Direct_Name;

   procedure Set_Entry_Index
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Statement);
      Set_Node5_With_Parent (N, Val);
   end Set_Entry_Index;

   procedure Set_Entry_Index_Specification
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Body_Formal_Part);
      Set_Node1_With_Parent (N, Val);
   end Set_Entry_Index_Specification;

   procedure Set_Error_Posted
      (N : Node_Id; Val : Boolean := True) is
   begin
      Set_Flag13 (N, Val);
   end Set_Error_Posted;

   procedure Set_Etype
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Has_Etype);
      Set_Node5 (N, Val); -- Sem field, don't set parent
   end Set_Etype;

   procedure Set_Evaluate_Once
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      Set_Flag10 (N, Val);
   end Set_Evaluate_Once;

   procedure Set_Exception_Choices
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Exception_Handler);
      Set_List4_With_Parent (N, Val);
   end Set_Exception_Choices;

   procedure Set_Exception_Handlers
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Handled_Sequence_Of_Statements);
      Set_List4_With_Parent (N, Val);
   end Set_Exception_Handlers;

   procedure Set_Explicit_Generic_Actual_Parameter
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Generic_Association);
      Set_Node1_With_Parent (N, Val);
   end Set_Explicit_Generic_Actual_Parameter;

   procedure Set_Expression
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Allocator
          or else NT (N).Nkind = N_Assignment_Statement
          or else NT (N).Nkind = N_At_Clause
          or else NT (N).Nkind = N_Attribute_Definition_Clause
          or else NT (N).Nkind = N_Case_Statement
          or else NT (N).Nkind = N_Code_Statement
          or else NT (N).Nkind = N_Component_Association
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Delay_Relative_Statement
          or else NT (N).Nkind = N_Delay_Until_Statement
          or else NT (N).Nkind = N_Discriminant_Association
          or else NT (N).Nkind = N_Discriminant_Specification
          or else NT (N).Nkind = N_Expression_Actions
          or else NT (N).Nkind = N_Extension_Aggregate
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Free_Statement
          or else NT (N).Nkind = N_Mod_Clause
          or else NT (N).Nkind = N_Modular_Type_Definition
          or else NT (N).Nkind = N_Number_Declaration
          or else NT (N).Nkind = N_Object_Declaration
          or else NT (N).Nkind = N_Parameter_Specification
          or else NT (N).Nkind = N_Pragma_Argument_Association
          or else NT (N).Nkind = N_Qualified_Expression
          or else NT (N).Nkind = N_Return_Statement
          or else NT (N).Nkind = N_Type_Conversion
          or else NT (N).Nkind = N_Unchecked_Type_Conversion);
      Set_Node3_With_Parent (N, Val);
   end Set_Expression;

   procedure Set_Expressions
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Aggregate
          or else NT (N).Nkind = N_Attribute_Reference
          or else NT (N).Nkind = N_Concat_Multiple
          or else NT (N).Nkind = N_Conditional_Expression
          or else NT (N).Nkind = N_Indexed_Component);
      Set_List1_With_Parent (N, Val);
   end Set_Expressions;

   procedure Set_First_Bit
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Clause);
      Set_Node3_With_Parent (N, Val);
   end Set_First_Bit;

   procedure Set_First_Itype
      (N : Node_Id; Val : Entity_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Has_Itypes);
      Set_Node2 (N, Val);  -- No parent, this is a semantic field
   end Set_First_Itype;

   procedure Set_First_Name
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      Set_Flag5 (N, Val);
   end Set_First_Name;

   procedure Set_First_Named_Actual
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Call_Statement
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Procedure_Call_Statement);
      Set_Node4 (N, Val); -- Note: semantic node, parent pointer not set
   end Set_First_Named_Actual;

   procedure Set_Following_Pragmas
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      Set_List3_With_Parent (N, Val);
   end Set_Following_Pragmas;

   procedure Set_Formal_Type_Definition
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Type_Declaration);
      Set_Node3_With_Parent (N, Val);
   end Set_Formal_Type_Definition;

   procedure Set_Generic_Associations
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Package_Declaration
          or else NT (N).Nkind = N_Function_Instantiation
          or else NT (N).Nkind = N_Package_Instantiation
          or else NT (N).Nkind = N_Procedure_Instantiation);
      Set_List3_With_Parent (N, Val);
   end Set_Generic_Associations;

   procedure Set_Generic_Formal_Declarations
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Generic_Package_Declaration
          or else NT (N).Nkind = N_Generic_Subprogram_Declaration);
      Set_List3_With_Parent (N, Val);
   end Set_Generic_Formal_Declarations;

   procedure Set_Generic_Parent
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Function_Specification
          or else NT (N).Nkind = N_Package_Specification
          or else NT (N).Nkind = N_Procedure_Specification);
      Set_Node5 (N, Val);
   end Set_Generic_Parent;

   procedure Set_Handled_Statement_Sequence
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Statement
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Entry_Body
          or else NT (N).Nkind = N_Package_Body
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Task_Body);
      Set_Node4_With_Parent (N, Val);
   end Set_Handled_Statement_Sequence;

   procedure Set_Has_Created_Identifier
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Loop_Statement);
      Set_Flag1 (N, Val);
   end Set_Has_Created_Identifier;

   procedure Set_Has_Dynamic_Itype
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Has_Itypes);
      Set_Flag14 (N, Val);
   end Set_Has_Dynamic_Itype;

   procedure Set_Has_No_Side_Effects
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      Set_Flag8 (N, Val);
   end Set_Has_No_Side_Effects;

   procedure Set_Has_Priority_Pragma
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Protected_Definition
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Task_Definition);
      Set_Flag6 (N, Val);
   end Set_Has_Priority_Pragma;

   procedure Set_Has_Private_View
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
         or else NT (N).Nkind in N_Op
         or else NT (N).Nkind = N_Character_Literal
         or else NT (N).Nkind = N_Expanded_Name
         or else NT (N).Nkind = N_Identifier
         or else NT (N).Nkind = N_Operator_Symbol);
      Set_Flag11 (N, Val);
   end Set_Has_Private_View;

   procedure Set_Has_Storage_Size_Pragma
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Task_Definition);
      Set_Flag5 (N, Val);
   end Set_Has_Storage_Size_Pragma;

   procedure Set_Has_Unknown_Discriminants
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Type_Declaration
          or else NT (N).Nkind = N_Incomplete_Type_Declaration
          or else NT (N).Nkind = N_Private_Extension_Declaration
          or else NT (N).Nkind = N_Private_Type_Declaration);
      Set_Flag3 (N, Val);
   end Set_Has_Unknown_Discriminants;

   procedure Set_High_Bound
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Range
          or else NT (N).Nkind = N_Real_Range_Specification
          or else NT (N).Nkind = N_Signed_Integer_Type_Definition);
      Set_Node2_With_Parent (N, Val);
   end Set_High_Bound;

   procedure Set_Homonym
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Defining_Character_Literal
          or else NT (N).Nkind = N_Defining_Identifier
          or else NT (N).Nkind = N_Defining_Operator_Symbol);
      Set_Node4 (N, Val); -- no parent, semantic field
   end Set_Homonym;

   procedure Set_Identifier
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_At_Clause
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Designator
          or else NT (N).Nkind = N_Enumeration_Representation_Clause
          or else NT (N).Nkind = N_Handled_Sequence_Of_Statements
          or else NT (N).Nkind = N_Label
          or else NT (N).Nkind = N_Loop_Statement
          or else NT (N).Nkind = N_Record_Representation_Clause);
      Set_Node1_With_Parent (N, Val);
   end Set_Identifier;

   procedure Set_Implicit_With
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      Set_Flag2 (N, Val);
   end Set_Implicit_With;

   procedure Set_In_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Parameter_Specification);
      Set_Flag1 (N, Val);
   end Set_In_Present;

   procedure Set_Intval
      (N : Node_Id; Val : Uint) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Integer_Literal);
      Set_Uint3 (N, Val);
   end Set_Intval;

   procedure Set_Is_Folded
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      Set_Flag7 (N, Val);
   end Set_Is_Folded;

   procedure Set_Is_Overloaded
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      Set_Flag5 (N, Val);
   end Set_Is_Overloaded;

   procedure Set_Is_Static
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Subexpr);
      Set_Flag6 (N, Val);
   end Set_Is_Static;

   procedure Set_Is_Task_Master
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Block_Statement
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Task_Body);
      Set_Flag5 (N, Val);
   end Set_Is_Task_Master;

   procedure Set_Iteration_Scheme
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Loop_Statement);
      Set_Node2_With_Parent (N, Val);
   end Set_Iteration_Scheme;

   procedure Set_Label
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Implicit_Label_Declaration);
      Set_Node2 (N, Val);
   end Set_Label;

   procedure Set_Last_Bit
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Clause);
      Set_Node4_With_Parent (N, Val);
   end Set_Last_Bit;

   procedure Set_Last_Name
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_With_Clause);
      Set_Flag6 (N, Val);
   end Set_Last_Name;

   procedure Set_Library_Unit
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit
          or else NT (N).Nkind = N_Package_Body_Stub
          or else NT (N).Nkind = N_Protected_Body_Stub
          or else NT (N).Nkind = N_Subprogram_Body_Stub
          or else NT (N).Nkind = N_Task_Body_Stub
          or else NT (N).Nkind = N_With_Clause);
      Set_Node4 (N, Val); -- No parent, this is a library field
   end Set_Library_Unit;

   procedure Set_Left_Opnd
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Binary_Op);
      Set_Node2_With_Parent (N, Val);
   end Set_Left_Opnd;

   procedure Set_Literals
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Enumeration_Type_Definition);
      Set_List1_With_Parent (N, Val);
   end Set_Literals;

   procedure Set_Limited_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Private_Type_Definition
          or else NT (N).Nkind = N_Private_Type_Declaration
          or else NT (N).Nkind = N_Record_Definition);
      Set_Flag2 (N, Val);
   end Set_Limited_Present;

   procedure Set_Loop_Parameter_Specification
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Iteration_Scheme);
      Set_Node1_With_Parent (N, Val);
   end Set_Loop_Parameter_Specification;

   procedure Set_Low_Bound
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Range
          or else NT (N).Nkind = N_Real_Range_Specification
          or else NT (N).Nkind = N_Signed_Integer_Type_Definition);
      Set_Node1_With_Parent (N, Val);
   end Set_Low_Bound;

   procedure Set_Mod_Clause
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Record_Representation_Clause);
      Set_Node2_With_Parent (N, Val);
   end Set_Mod_Clause;

   procedure Set_More_Ids
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Discriminant_Specification
          or else NT (N).Nkind = N_Exception_Declaration
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Number_Declaration
          or else NT (N).Nkind = N_Object_Declaration
          or else NT (N).Nkind = N_Parameter_Specification);
      Set_Flag5 (N, Val);
   end Set_More_Ids;

   procedure Set_Name
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Assignment_Statement
          or else NT (N).Nkind = N_Attribute_Definition_Clause
          or else NT (N).Nkind = N_Defining_Program_Unit_Name
          or else NT (N).Nkind = N_Designator
          or else NT (N).Nkind = N_Entry_Call_Statement
          or else NT (N).Nkind = N_Exception_Renaming_Declaration
          or else NT (N).Nkind = N_Exit_Statement
          or else NT (N).Nkind = N_Formal_Package_Declaration
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Function_Instantiation
          or else NT (N).Nkind = N_Generic_Function_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Procedure_Renaming_Declaration
          or else NT (N).Nkind = N_Goto_Statement
          or else NT (N).Nkind = N_Object_Renaming_Declaration
          or else NT (N).Nkind = N_Package_Instantiation
          or else NT (N).Nkind = N_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Procedure_Call_Statement
          or else NT (N).Nkind = N_Procedure_Instantiation
          or else NT (N).Nkind = N_Raise_Statement
          or else NT (N).Nkind = N_Requeue_Statement
          or else NT (N).Nkind = N_Subprogram_Renaming_Declaration
          or else NT (N).Nkind = N_Subunit
          or else NT (N).Nkind = N_Variant_Part
          or else NT (N).Nkind = N_With_Clause);
      Set_Node2_With_Parent (N, Val);
   end Set_Name;

   procedure Set_Names
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Abort_Statement
          or else NT (N).Nkind = N_Use_Package_Clause);
      Set_List2_With_Parent (N, Val);
   end Set_Names;

   procedure Set_Next_Entity
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Defining_Character_Literal
          or else NT (N).Nkind = N_Defining_Identifier
          or else NT (N).Nkind = N_Defining_Operator_Symbol);
      Set_Node2 (N, Val); -- no parent, semantic field
   end Set_Next_Entity;

   procedure Set_Next_Named_Actual
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Parameter_Association);
      Set_Node4 (N, Val); -- Note: semantic node, no parent pointer needed
   end Set_Next_Named_Actual;

   procedure Set_No_Defer
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Raise_Statement);
      Set_Flag1 (N, Val);
   end Set_No_Defer;

   procedure Set_Null_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_List
          or else NT (N).Nkind = N_Record_Definition);
      Set_Flag3 (N, Val);
   end Set_Null_Present;

   procedure Set_Null_Record_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Aggregate
          or else NT (N).Nkind = N_Extension_Aggregate);
      Set_Flag2 (N, Val);
   end Set_Null_Record_Present;

   procedure Set_Numerator
      (N : Node_Id; Val : Uint) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Real_Literal);
      Set_Uint3 (N, Val);
   end Set_Numerator;

   procedure Set_Object_Definition
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Object_Declaration);
      Set_Node4_With_Parent (N, Val);
   end Set_Object_Definition;

   procedure Set_Others_Discrete_Choices
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Others_Choice);
      Set_List1_With_Parent (N, Val);
   end Set_Others_Discrete_Choices;

   procedure Set_Out_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Parameter_Specification);
      Set_Flag2 (N, Val);
   end Set_Out_Present;

   procedure Set_Parameter_Associations
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Entry_Call_Statement
          or else NT (N).Nkind = N_Function_Call
          or else NT (N).Nkind = N_Procedure_Call_Statement);
      Set_List3_With_Parent (N, Val);
   end Set_Parameter_Associations;

   procedure Set_Parameter_Specifications
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Accept_Statement
          or else NT (N).Nkind = N_Access_Function_Definition
          or else NT (N).Nkind = N_Access_Procedure_Definition
          or else NT (N).Nkind = N_Entry_Body_Formal_Part
          or else NT (N).Nkind = N_Entry_Declaration
          or else NT (N).Nkind = N_Function_Specification
          or else NT (N).Nkind = N_Procedure_Specification);
      Set_List3_With_Parent (N, Val);
   end Set_Parameter_Specifications;

   procedure Set_Parameter_Type
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Parameter_Specification);
      Set_Node2_With_Parent (N, Val);
   end Set_Parameter_Type;

   procedure Set_Parent_Spec
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Function_Instantiation
          or else NT (N).Nkind = N_Generic_Function_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Package_Declaration
          or else NT (N).Nkind = N_Generic_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Procedure_Renaming_Declaration
          or else NT (N).Nkind = N_Generic_Subprogram_Declaration
          or else NT (N).Nkind = N_Package_Declaration
          or else NT (N).Nkind = N_Package_Instantiation
          or else NT (N).Nkind = N_Package_Renaming_Declaration
          or else NT (N).Nkind = N_Procedure_Instantiation
          or else NT (N).Nkind = N_Subprogram_Declaration
          or else NT (N).Nkind = N_Subprogram_Renaming_Declaration);
      Set_Node4 (N, Val); -- semantic node, no parent pointer
   end Set_Parent_Spec;

   procedure Set_Position
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Clause);
      Set_Node2 (N, Val); -- semantic node, no parent pointer
   end Set_Position;

   procedure Set_Pragma_Argument_Associations
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Pragma);
      Set_List2_With_Parent (N, Val);
   end Set_Pragma_Argument_Associations;

   procedure Set_Preelaborable
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      Set_Flag2 (N, Val);
   end Set_Preelaborable;

   procedure Set_Prefix
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Attribute_Reference
          or else NT (N).Nkind = N_Expanded_Name
          or else NT (N).Nkind = N_Explicit_Dereference
          or else NT (N).Nkind = N_Indexed_Component
          or else NT (N).Nkind = N_Reference
          or else NT (N).Nkind = N_Selected_Component
          or else NT (N).Nkind = N_Slice);
      Set_Node3_With_Parent (N, Val);
   end Set_Prefix;

   procedure Set_Prev_Ids
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Discriminant_Specification
          or else NT (N).Nkind = N_Exception_Declaration
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Number_Declaration
          or else NT (N).Nkind = N_Object_Declaration
          or else NT (N).Nkind = N_Parameter_Specification);
      Set_Flag6 (N, Val);
   end Set_Prev_Ids;

   procedure Set_Private_Declarations
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Package_Specification
          or else NT (N).Nkind = N_Protected_Definition
          or else NT (N).Nkind = N_Task_Definition);
      Set_List4_With_Parent (N, Val);
   end Set_Private_Declarations;

   procedure Set_Private_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit
          or else NT (N).Nkind = N_Formal_Derived_Type_Definition);
      Set_Flag1 (N, Val);
   end Set_Private_Present;

   procedure Set_Procedure_To_Call
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Allocator
          or else NT (N).Nkind = N_Free_Statement);
      Set_Node4_With_Parent (N, Val);
   end Set_Procedure_To_Call;

   procedure Set_Proper_Body
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Subunit);
      Set_Node1_With_Parent (N, Val);
   end Set_Proper_Body;

   procedure Set_Protected_Definition
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Protected_Type_Declaration
          or else NT (N).Nkind = N_Single_Protected_Declaration);
      Set_Node3_With_Parent (N, Val);
   end Set_Protected_Definition;

   procedure Set_Protected_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Access_Function_Definition
          or else NT (N).Nkind = N_Access_Procedure_Definition);
      Set_Flag1 (N, Val);
   end Set_Protected_Present;

   procedure Set_Range_Constraint
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Delta_Constraint
          or else NT (N).Nkind = N_Digits_Constraint);
      Set_Node4_With_Parent (N, Val);
   end Set_Range_Constraint;

   procedure Set_Range_Expression
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Range_Constraint);
      Set_Node4_With_Parent (N, Val);
   end Set_Range_Expression;

   procedure Set_Real_Range_Specification
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Decimal_Fixed_Point_Definition
          or else NT (N).Nkind = N_Floating_Point_Definition
          or else NT (N).Nkind = N_Ordinary_Fixed_Point_Definition);
      Set_Node4_With_Parent (N, Val);
   end Set_Real_Range_Specification;

   procedure Set_Record_Extension_Part
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Derived_Type_Definition);
      Set_Node3_With_Parent (N, Val);
   end Set_Record_Extension_Part;

   procedure Set_Redundant_Use
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Expanded_Name
          or else NT (N).Nkind = N_Identifier);
      Set_Flag3 (N, Val);
   end Set_Redundant_Use;

   procedure Set_Return_Type
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Return_Statement);
      Set_Node1 (N, Val); -- semantic node, no parent set
   end Set_Return_Type;

   procedure Set_Reverse_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Loop_Parameter_Specification);
      Set_Flag1 (N, Val);
   end Set_Reverse_Present;

   procedure Set_Right_Opnd
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind in N_Op);
      Set_Node3_With_Parent (N, Val);
   end Set_Right_Opnd;

   procedure Set_Scope
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Defining_Character_Literal
          or else NT (N).Nkind = N_Defining_Identifier
          or else NT (N).Nkind = N_Defining_Operator_Symbol);
      Set_Node3 (N, Val); -- no parent, semantic field
   end Set_Scope;

   procedure Set_Selective_Accept_Alternatives
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Selective_Accept);
      Set_List1_With_Parent (N, Val);
   end Set_Selective_Accept_Alternatives;

   procedure Set_Selector_Name
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Expanded_Name
          or else NT (N).Nkind = N_Generic_Association
          or else NT (N).Nkind = N_Parameter_Association
          or else NT (N).Nkind = N_Selected_Component);
      Set_Node2_With_Parent (N, Val);
   end Set_Selector_Name;

   procedure Set_Selector_Names
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Discriminant_Association);
      Set_List1_With_Parent (N, Val);
   end Set_Selector_Names;

   procedure Set_Specification
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Abstract_Subprogram_Declaration
          or else NT (N).Nkind = N_Formal_Subprogram_Declaration
          or else NT (N).Nkind = N_Generic_Package_Declaration
          or else NT (N).Nkind = N_Generic_Subprogram_Declaration
          or else NT (N).Nkind = N_Package_Declaration
          or else NT (N).Nkind = N_Subprogram_Body
          or else NT (N).Nkind = N_Subprogram_Body_Stub
          or else NT (N).Nkind = N_Subprogram_Declaration
          or else NT (N).Nkind = N_Subprogram_Renaming_Declaration);
      Set_Node1_With_Parent (N, Val);
   end Set_Specification;

   procedure Set_Statements
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Abortable_Part
          or else NT (N).Nkind = N_Accept_Alternative
          or else NT (N).Nkind = N_Case_Statement_Alternative
          or else NT (N).Nkind = N_Delay_Alternative
          or else NT (N).Nkind = N_Entry_Call_Alternative
          or else NT (N).Nkind = N_Exception_Handler
          or else NT (N).Nkind = N_Handled_Sequence_Of_Statements
          or else NT (N).Nkind = N_Loop_Statement
          or else NT (N).Nkind = N_Triggering_Alternative);
      Set_List3_With_Parent (N, Val);
   end Set_Statements;

   procedure Set_Storage_Pool
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Allocator
          or else NT (N).Nkind = N_Free_Statement);
      Set_Node1_With_Parent (N, Val);
   end Set_Storage_Pool;

   procedure Set_Strval
      (N : Node_Id; Val : String_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Operator_Symbol
          or else NT (N).Nkind = N_String_Literal);
      Set_Str3 (N, Val);
   end Set_Strval;

   procedure Set_Subtype_Indication
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Access_To_Object_Definition
          or else NT (N).Nkind = N_Component_Declaration
          or else NT (N).Nkind = N_Constrained_Array_Definition
          or else NT (N).Nkind = N_Derived_Type_Definition
          or else NT (N).Nkind = N_Private_Extension_Declaration
          or else NT (N).Nkind = N_Subtype_Declaration
          or else NT (N).Nkind = N_Unconstrained_Array_Definition);
      Set_Node5_With_Parent (N, Val);
   end Set_Subtype_Indication;

   procedure Set_Subtype_Mark
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Access_Definition
          or else NT (N).Nkind = N_Access_Function_Definition
          or else NT (N).Nkind = N_Formal_Derived_Type_Definition
          or else NT (N).Nkind = N_Formal_Object_Declaration
          or else NT (N).Nkind = N_Function_Specification
          or else NT (N).Nkind = N_Object_Renaming_Declaration
          or else NT (N).Nkind = N_Qualified_Expression
          or else NT (N).Nkind = N_Subtype_Indication
          or else NT (N).Nkind = N_Type_Conversion
          or else NT (N).Nkind = N_Unchecked_Type_Conversion);
      Set_Node4_With_Parent (N, Val);
   end Set_Subtype_Mark;

   procedure Set_Subtype_Marks
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Unconstrained_Array_Definition
          or else NT (N).Nkind = N_Use_Type_Clause);
      Set_List2_With_Parent (N, Val);
   end Set_Subtype_Marks;

   procedure Set_Tagged_Present
      (N : Node_Id; Val : Boolean := True) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Formal_Private_Type_Definition
          or else NT (N).Nkind = N_Private_Type_Declaration
          or else NT (N).Nkind = N_Record_Definition);
      Set_Flag1 (N, Val);
   end Set_Tagged_Present;

   procedure Set_Task_Definition
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Single_Task_Declaration
          or else NT (N).Nkind = N_Task_Type_Declaration);
      Set_Node3_With_Parent (N, Val);
   end Set_Task_Definition;

   procedure Set_Then_Statements
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Elsif_Part
          or else NT (N).Nkind = N_If_Statement);
      Set_List2_With_Parent (N, Val);
   end Set_Then_Statements;

   procedure Set_Triggering_Alternative
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Asynchronous_Select);
      Set_Node1_With_Parent (N, Val);
   end Set_Triggering_Alternative;

   procedure Set_Triggering_Statement
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Triggering_Alternative);
      Set_Node1_With_Parent (N, Val);
   end Set_Triggering_Statement;

   procedure Set_Type_Definition
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Full_Type_Declaration);
      Set_Node3_With_Parent (N, Val);
   end Set_Type_Definition;

   procedure Set_Unit
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Compilation_Unit);
      Set_Node2_With_Parent (N, Val);
   end Set_Unit;

   procedure Set_Variant_Part
      (N : Node_Id; Val : Node_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Component_List);
      Set_Node4_With_Parent (N, Val);
   end Set_Variant_Part;

   procedure Set_Variants
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Variant_Part);
      Set_List1_With_Parent (N, Val);
   end Set_Variants;

   procedure Set_Visible_Declarations
      (N : Node_Id; Val : List_Id) is
   begin
      pragma Assert (False
          or else NT (N).Nkind = N_Package_Specification
          or else NT (N).Nkind = N_Protected_Definition
          or else NT (N).Nkind = N_Task_Definition);
      Set_List2_With_Parent (N, Val);
   end Set_Visible_Declarations;

end Sinfo;
